from __future__ import annotations

from abc import ABC, abstractmethod
from datetime import datetime
from functools import cached_property
from typing import Union, Type

from .botcontext import BotContext
from .botmsg import BotMessage
from .user import User


class ReplyToMessage(ABC):
    __slots__ = ('_data', '__dict__')

    def __init__(self, data, inbound):
        self._data = data
        self._inbound = inbound

    def get(self, key, default_value=None):
        return self._data.get(key, default_value)

    @property
    @abstractmethod
    def text(self) -> Union[str, None]:
        pass

    @property
    @abstractmethod
    def message_id(self) -> Union[int, str]:
        pass

    @property
    @abstractmethod
    def chat_id(self) -> Union[int, str]:
        pass

    @property
    @abstractmethod
    def user_id(self) -> Union[int, str]:
        pass

    @property
    @abstractmethod
    def user_visual_name(self) -> Union[str, None]:
        pass

    @property
    @abstractmethod
    def date_raw(self) -> Union[int, None]:
        pass

    @cached_property
    def date(self) -> Union[datetime, None]:
        return self.inbound.raw_date_converter(self.date_raw)

    @property
    @abstractmethod
    def inbound(self) -> InboundMessage:
        pass


class InboundMessage(ABC):
    __slots__ = ('_data', '__dict__', '__channel_type')

    _UserClass = User
    _ReplyMessageClass = ReplyToMessage

    def __init__(self, data):
        self._data = data
        self.validate()
        self._user = self._UserClass(self.user_id)
        self._context = None

    def __getitem__(self, key):
        return self._data[key]

    @property
    @abstractmethod
    def text(self) -> str:
        pass

    # region User
    @property
    def user(self) -> User:
        return self._user

    @property
    @abstractmethod
    def user_id(self) -> Union[int, str]:
        pass

    @property
    @abstractmethod
    def user_id_url(self) -> str:
        pass

    @property
    def user_visual_name(self) -> str:
        return self.user.visual_name or self.user.short_name

    @property
    def user_data(self):
        return None

    # endregion

    @property
    @abstractmethod
    def date_raw(self) -> Union[int, None]:
        pass

    @cached_property
    def date(self) -> Union[datetime, None]:
        return self.raw_date_converter(self.date_raw)

    def raw_date_converter(self, timestamp: int) -> Union[datetime, None]:
        """Default raw_date_converter_that can be overriden."""
        if timestamp:
            return datetime.fromtimestamp(timestamp)
        return None

    @property
    @abstractmethod
    def chat_id(self) -> Union[int, str]:
        pass

    @property
    @abstractmethod
    def message_id(self) -> Union[int, str]:
        pass

    @property
    def is_private(self) -> bool:
        return self.user_id == self.chat_id

    def get(self, key, default_value):
        return self._data.get(key, default_value)

    @property
    @abstractmethod
    def secret_code(self) -> str:
        pass

    @cached_property
    def reply_to_msg(self) -> Union[ReplyToMessage, None]:
        return self._ReplyMessageClass(self.reply_to_msg_raw, self)

    @property
    @abstractmethod
    def reply_to_msg_raw(self) -> Union[dict, None]:
        pass

    def validate(self):
        assert self.user_id is not None

    @property
    def context(self) -> BotContext:
        return self._context


class InboundProcessor(ABC):
    error_message_default = 'Inbound processing error'

    @staticmethod
    @abstractmethod
    def process(context: BotContext, inbound: InboundMessage):
        """context.replies.add(BotMessage(inbound.text))"""
        pass


class DefaultInboundProcessor(InboundProcessor):
    """Empty processor"""

    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        pass


class EchoInboundProcessor(InboundProcessor):
    """Echo processor"""

    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        context.replies.add(BotMessage(inbound.text))


class EchoReplyToInboundProcessor(InboundProcessor):
    """Echo processor"""

    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        context.replies.add(BotMessage(inbound.text, is_reply_to_message=True))


class EchoNameInboundProcessor(InboundProcessor):
    """Echo processor"""

    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        # text = f'{context.bot.get_user_visual_name(inbound)} is your name.'
        text = f'{inbound.user.visual_name} is your name.'
        context.replies.add(BotMessage(text))


class InboundMessageFactory:
    __slots__ = ('_default_inbound_message_class',)

    def __init__(self, default_inbound_message_class: Type[InboundMessage]):
        self._default_inbound_message_class = default_inbound_message_class

    def create(self, data) -> InboundMessage:
        inbound_message_class = self.get_inbound_message_class(data)
        return inbound_message_class(data)

    def get_inbound_message_class(self, data) -> Type[InboundMessage]:
        return self._default_inbound_message_class



from functools import cached_property


class BotConfig:
    __slots__ = ('_data', '__dict__')

    def __init__(self, data):
        self._data = data

    def __getitem__(self, key):
        return self._data[key]

    @property
    def token(self) -> str:
        return self['token']

    @property
    def webhook_url(self) -> str:
        return self['webhook_url']

    @property
    def webhook_secret(self):
        return self.get('webhook_secret', None)

    @property
    def is_send_process_error_message(self):
        """Send error message on processing error step"""
        return self.get('is_send_process_error_message', True)

    @property
    def bot_id(self):
        return self['bot_id']

    @cached_property
    def bot_id_string(self):
        return str(self.bot_id)

    @property
    def name(self) -> str:
        return self._data.get('name', '')


    def get(self, key, default_value=None):
        return self._data.get(key, default_value)



from __future__ import annotations

from abc import ABC
from typing import TYPE_CHECKING

from .botdebug import BotHandleResult
from .botmsg import BotReplies

if TYPE_CHECKING:
    from .botadapter import UBotAdapter
    from .inbound import InboundMessage


class BotContext(ABC):
    __slots__ = ('_replies', '_inbound', '_bot', '_handle_result')

    def __init__(self, bot: UBotAdapter):
        self._inbound = None
        self._bot = bot
        self._replies = BotReplies()
        self._handle_result = BotHandleResult()

    @property
    def replies(self) -> BotReplies:
        return self._replies

    @property
    def bot(self) -> UBotAdapter:
        return self._bot

    @property
    def bot_info(self):
        return self.bot.info

    @property
    def inbound(self) -> InboundMessage:
        return self._inbound

    @inbound.setter
    def inbound(self, value: InboundMessage):
        value._context = self
        self._inbound = value

    @property
    def handle_result(self) -> BotHandleResult:
        return self._handle_result

    def clear(self):
        self.replies.clear()
        self._handle_result.__init__()



from typing import List, Type, Callable

import ubotadapter
from ubotadapter import InboundMessage
from ubotadapter.botcontext import BotContext
from ubotadapter.bots import *
from ubotadapter.inbound import InboundProcessor
from .test_data import *


class BadInboundProcessor(InboundProcessor):

    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        raise ValueError("Pseudo-processing error")


def get_tg_bot(**config):
    with open(os.path.join(DATA_DIR, 'telegram.json'), 'r', encoding='utf-8') as f:
        config.update(json.load(f))
    return Telegram(config)


def get_dtf_bot(**config):
    with open(os.path.join(DATA_DIR, 'dtf.json'), 'r', encoding='utf-8') as f:
        config.update(json.load(f))
    return DTF(config)


def get_vk_bot():
    return VK(os.path.join(DATA_DIR, 'vk.json'))


class BotTestData:
    def __init__(self,
                 bot: Callable,
                 channel: ubotadapter.ChannelType,
                 message_getter: TestMessage,
                 visual_name_text: str,
                 all_message_getters: List[TestMessage]):
        self._bot = bot
        self.channel = channel
        self.message_getter = message_getter
        self.visual_name_text = visual_name_text
        self.all_message_getters = all_message_getters

    @property
    def bot(self) -> ubotadapter.UBotAdapter:
        return self._bot()


BOTS_LIBRARY: List[BotTestData] = [
    BotTestData(get_tg_bot, Telegram.channel, TGTestMessage(), 'Igor Shephard / Igor',
                [TGTestMessage(), TGCallbackTestMessage()]),

    BotTestData(get_vk_bot, VK.channel, VKTestMessage(), 'Игорь Шепард / Игорь', [VKTestMessage()]),

    BotTestData(get_dtf_bot, DTF.channel, DTFTestMessage(), 'Igor Shephard / Igor Shephard', [DTFTestMessage()])
]



from __future__ import annotations

from datetime import datetime
from functools import cached_property
from typing import Type, Union

from ubotadapter import InboundMessage, InboundMessageFactory, ReplyToMessage
from .user import TelegramUser


class TGReplyToMessage(ReplyToMessage):
    @property
    def inbound(self) -> TGInboundMessage:
        return self._inbound

    @cached_property
    def _from(self) -> dict:
        return self.get('from')

    @property
    def text(self) -> Union[str, None]:
        return self.get('text')

    @property
    def message_id(self) -> Union[int, str]:
        return self.get('message_id')

    @property
    def chat_id(self) -> Union[int, str]:
        return self.get('chat', {}).get('id', None)

    @property
    def user_id(self) -> Union[int, str]:
        return self._from.get('id', None)

    @property
    def user_visual_name(self) -> Union[str, None]:
        return self._from.get('username', None)

    @property
    def date_raw(self) -> Union[int, None]:
        return self.get('date', None)


class TGInboundMessage(InboundMessage):
    _UserClass = TelegramUser
    _ReplyMessageClass = TGReplyToMessage

    @property
    def message(self) -> dict:
        return self['message']

    @property
    def user_data(self) -> dict:
        return self._from

    @property
    def _from(self) -> dict:
        return self.message['from']

    @property
    def text_only(self) -> str:
        return self.message.get('text', '')

    @property
    def sticker(self) -> dict:
        return self.message.get('sticker', {})

    @property
    def text(self) -> str:
        return self.message.get('text', self.sticker.get('emoji', ''))

    @property
    def user_id(self) -> int:
        return self._from['id']

    @property
    def user_id_url(self) -> str:
        return f'@{self._from["username"]}'

    @property
    def chat_id(self) -> str:
        return self.message['chat']['id']

    @property
    def message_id(self) -> str:
        return self.message['message_id']

    @property
    def secret_code(self) -> None:
        return None

    @property
    def is_private(self) -> bool:
        return self.message.get('type', None) == 'private'

    @property
    def date_raw(self) -> Union[int, None]:
        return self.message.get('date', None)

    @property
    def reply_to_msg_raw(self) -> Union[dict, None]:
        return self.message.get('reply_to_message', {})


class TGInboundCallbackQuery(TGInboundMessage):
    @property
    def callback_query(self):
        return self['callback_query']

    @property
    def message(self):
        return self.callback_query['message']

    @property
    def _from(self):
        return self.callback_query['from']

    @property
    def text(self) -> str:
        data = self.callback_query['data']
        if isinstance(data, str):
            return data
        return ''


class TGInboundMessageFactory(InboundMessageFactory):
    def get_inbound_message_class(self, data) -> Type[InboundMessage]:
        if 'callback_query' in data:
            return TGInboundCallbackQuery
        return self._default_inbound_message_class



from ubotadapter.bots import VK
from ubotadapter.inbound import EchoInboundProcessor, EchoReplyToInboundProcessor


def get_bot():
    bot = VK('data/vk.json')
    bot.set_inbound_processor(EchoInboundProcessor)
    return bot


def test_msg():
    bot = get_bot()
    bot.handle({
        'type': 'message_new',
        'object': {
            'message': {
                'date': 1624210273,
                'from_id': 4909962,
                'id': 1,
                'out': 0,
                'peer_id': 4909962,
                'text': 'test_ubot',
                'conversation_message_id': 1,
                'fwd_messages': [],
                'important': False,
                'random_id': 0,
                'attachments': [],
                'is_hidden': False
            },
            'client_info': {
                'button_actions': ['text', 'vkpay', 'open_app', 'location', 'open_link', 'callback', 'intent_subscribe',
                                   'intent_unsubscribe'],
                'keyboard': True,
                'inline_keyboard': True,
                'carousel': True,
                'lang_id': 0
            }
        },
        'group_id': 204429470,
        'event_id': 'b6ca7d45d8055839911ed53a23f46b8098fe526f'
    }
    )


def test_msg_groupchat():
    bot = get_bot()
    bot.handle({
        'type': 'message_new',
        'object': {
            'message': {
                'date': 1624208087,
                'from_id': 4909962,
                'id': 1,
                'out': 0,
                'peer_id': 2000000001,
                'text': 'qwert',
                'conversation_message_id': 7,
                'fwd_messages': [],
                'important': False,
                'random_id': 0,
                'attachments': [],
                'is_hidden': False
            },
            'client_info': {
                'button_actions': ['text', 'vkpay', 'open_app', 'location', 'open_link', 'callback', 'intent_subscribe',
                                   'intent_unsubscribe'],
                'keyboard': True,
                'inline_keyboard': True,
                'carousel': True,
                'lang_id': 0
            }
        },
        'group_id': 204429470,
        'event_id': '78a6cd66ea3b22fb218b9fb7f969868fa85220ae'
    }
    )


if __name__ == '__main__':
    import logging
    from ubotadapter.logger import logger

    logFormatter = logging.Formatter("%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s")

    # fileHandler = logging.FileHandler("{0}/{1}.log".format(logPath, fileName))
    # fileHandler.setFormatter(logFormatter)
    handler = logging.StreamHandler()
    handler.setFormatter(logFormatter)
    logger.addHandler(handler)
    logger.setLevel(logging.DEBUG)
    test_msg()
    # test_msg_groupchat()



from abc import ABC, abstractmethod
from typing import List, Type, Union

from .inbound import InboundMessage, BotContext


class MessageFilter(ABC):
    __slots__ = ()

    @staticmethod
    @abstractmethod
    def is_true(context: BotContext, message: InboundMessage) -> bool:
        pass


class NotBotMessage(MessageFilter):
    @staticmethod
    def is_true(context: BotContext, message: InboundMessage) -> bool:
        return message.user_id != context.bot.config.bot_id


class MessageFilterManager:
    __slots__ = ('_message_filters',)

    def __init__(self, filters: List[Type[MessageFilter]]):
        self._message_filters = filters

    @property
    def message_filters(self) -> List[Type[MessageFilter]]:
        return self._message_filters

    def check(self, context: BotContext, message: InboundMessage) -> Union[Type[MessageFilter], None]:
        for msg_filter in self.message_filters:
            if not msg_filter.is_true(context, message):
                return msg_filter
        return None

    def add_filter(self, msg_filter: Type[MessageFilter]):
        self._message_filters.append(msg_filter)

    def remove_filters(self):
        self._message_filters = []



from enum import Enum


class BotReplyException(Exception):
    pass


# region Processing

class BotProcessStatus(Enum):
    ok = 0
    unknown_error = 1
    inbound_error = 2
    filter_not_passed = 10
    wrong_secret = 100


class BotProcessResult:
    __slots__ = ('status', 'data', 'exception')

    def __init__(self, status: BotProcessStatus, data=None, exception: Exception = None):
        self.status = status
        self.data = data
        self.exception = exception

    def __str__(self):
        return [getattr(self, k) for k in self.__slots__]


class BotHandleStatus(Enum):
    ok = 0
    error = 1
    partial_error = 2


class BotReplyStatus(Enum):
    ok = 0
    unknown_error = 1
    unknown_exception = 2
    no_permission = 90


class BotReplyResult:
    __slots__ = ('status', 'data', 'exception')

    def __init__(self, status: BotReplyStatus, data=None, exception: Exception = None):
        self.status = status
        self.data = data
        self.exception = exception

    def __str__(self):
        return [getattr(self, k) for k in self.__slots__]


class BotHandleResult:
    __slots__ = ('_process_result', '_replies_result')

    def __init__(self):
        self._process_result = None
        self._replies_result = []

    @property
    def process_result(self) -> BotProcessResult:
        return self._process_result

    @process_result.setter
    def process_result(self, value: BotProcessResult):
        self._process_result = value

    @property
    def reply_result(self) -> BotReplyResult:
        return self._replies_result[-1]

    @reply_result.setter
    def reply_result(self, value: BotReplyResult):
        self._replies_result.append(value)

    @property
    def status(self) -> BotHandleStatus:
        if self.process_result.status == BotProcessStatus.ok:
            if any(result.status != BotReplyStatus.ok for result in self._replies_result):
                return BotHandleStatus.partial_error
            return BotHandleStatus.ok
        return BotHandleStatus.error

    @property
    def is_ok(self) -> bool:
        return self.status == BotHandleStatus.ok

    @property
    def no_replies(self) -> bool:
        return len(self._replies_result) == 0

    def report(self):
        return {
            'result': self.is_ok,
            'process': self._process_result.__str__(),
            'replies': [r.__str__() for r in self._replies_result]
        }



import telegram
from telegram import InlineKeyboardButton, KeyboardButton, InlineKeyboardMarkup, ReplyKeyboardMarkup

from ubotadapter import UBotAdapter, BotInfo, ChannelType
from ubotadapter.botdebug import *
from ubotadapter.logger import logger
from .inbound import TGInboundMessage, TGInboundMessageFactory


class Telegram(UBotAdapter):
    channel = ChannelType.Telegram
    _info = BotInfo(channel=ChannelType.Telegram)
    _inbound_message_factory = TGInboundMessageFactory
    _InboundMessageClass = TGInboundMessage

    def _init_api(self):
        self._api = telegram.Bot(self.config.token)

    @property
    def api(self) -> telegram.Bot:
        return self._api

    def reply(self, context, message):
        def serialize_inline_button(text: str, callback_data: str = None, **kwargs):
            return InlineKeyboardButton(text=text, callback_data=callback_data or text, **kwargs)

        params = dict(chat_id=self.context.inbound.chat_id, text=message.text)
        # buttons
        if message.keyboard:
            if message.keyboard.in_line:
                keyboard = message.keyboard.serialize_buttons(serialize_inline_button)
                reply_markup = InlineKeyboardMarkup(keyboard)
            else:
                keyboard = message.keyboard.serialize_buttons(KeyboardButton)
                reply_markup = ReplyKeyboardMarkup(keyboard, one_time_keyboard=message.keyboard.one_time)

            params.update(reply_markup=reply_markup)
        # reply to
        if message.is_reply_to_message or message.reply_to_message:
            message_id = message.reply_to_message if message.reply_to_message else self.context.inbound.message_id
            params.update(reply_to_message_id=message_id)
        response = self.api.send_message(**params)
        if response and response.message_id:
            return True, BotReplyStatus.ok
        logger.error(f'Reply {self.channel} error: ' + response)
        return False, BotReplyStatus.unknown_error

    def check_webhook(self):
        info = self.api.get_webhook_info()
        is_ok = bool(info.url) and info.url == self.config.webhook_url
        return is_ok, info.url

    def delete_webhook(self) -> bool:
        return self.api.delete_webhook()

    def set_webhook(self) -> bool:
        return self.api.set_webhook(url=self.config.webhook_url)

    def disable_webhook(self) -> bool:
        return self.api.delete_webhook()

    def update_webhook(self) -> (bool, str):
        info = self.api.get_webhook_info()
        if info.url != self.config.webhook_url:
            if info.url:
                self.disable_webhook()
            self.set_webhook()
        return self.check_webhook()

    def reset_webhook(self) -> (bool, str):
        self.disable_webhook()
        self.set_webhook()
        return self.check_webhook()

    def get_user_visual_name(self, inbound: TGInboundMessage) -> str:
        return inbound.user_visual_name

    def get_user_data(self, inbound: TGInboundMessage):
        return inbound.user_data


if __name__ == '__main__':
    tgbot = Telegram('config/telegram.json')
    print(tgbot.check_webhook())



from vkale.items.user_items import UserItem

from ubotadapter.user import User


class VKUser(User):
    @property
    def data(self) -> UserItem:
        return super().data

    def get_visual_name(self) -> str:
        return f'{self.data.first_name} {self.data.last_name}'

    def get_short_name(self) -> str:
        return self.data.first_name

    @property
    def id_url(self) -> str:
        return f'@id{self.uid}'



from __future__ import annotations

from functools import cached_property
from typing import Union

from ubotadapter import InboundMessage
from ubotadapter.inbound import ReplyToMessage
from .user import VKUser


class VKReplyToMessage(ReplyToMessage):
    @property
    def text(self) -> Union[str, None]:
        return self.get('text')

    @property
    def message_id(self) -> Union[int, str]:
        return self.get('id')

    @property
    def vk_conversation_message_id(self) -> int:
        return self.get('conversation_message_id')

    @property
    def chat_id(self) -> Union[int, str]:
        return self.get('peer_id')

    @property
    def user_id(self) -> Union[int, str]:
        return self.get('from_id')

    @property
    def user_visual_name(self) -> Union[str, None]:
        return f'@{self.user_id}'

    @property
    def date_raw(self) -> Union[int, None]:
        return self.get('date')

    @property
    def inbound(self) -> VKInboundMessage:
        return self._inbound


class VKInboundMessage(InboundMessage):
    _UserClass = VKUser
    _ReplyMessageClass = VKReplyToMessage

    @property
    def object(self):
        return self['object']

    @property
    def message(self):
        return self.object['message']

    @property
    def group_id(self) -> int:
        return self.get('group_id', None)

    @property
    def text(self) -> str:
        return self.message['text']

    @property
    def user_id(self) -> str:
        return self.message['from_id']

    @property
    def user_id_url(self) -> str:
        return f'@id{self.user_id}'

    @property
    def chat_id(self) -> str:
        return self.message['peer_id']

    @property
    def message_id(self) -> int:
        return self.message['id']

    @property
    def vk_conversation_message_id(self) -> int:
        return self.message['conversation_message_id']

    @property
    def secret_code(self) -> str:
        return self.get('secret', None)

    @property
    def date_raw(self) -> Union[int, None]:
        return self.message.get('date', None)

    @cached_property
    def reply_to_msg_raw(self) -> Union[dict, None]:
        return self.message.get('reply_message', None)



import datetime
import unittest

from ubotadapter import BotsManager
from ubotadapter.botdebug import *
from ubotadapter.msgfilter import MessageFilter
from ubotadapter.test.test_tools import *
from .test_bot import TestingInboundProcessor, MARKER


class TestBotsInit(unittest.TestCase):

    def test_vk_bot(self):
        bot = get_vk_bot()
        self.assertIsNotNone(bot)

    def test_telegram_bot(self):
        bot = get_tg_bot()
        self.assertIsNotNone(bot)

    def test_dtf_bot(self):
        bot = get_dtf_bot()
        self.assertIsNotNone(bot)


class TestBotsInboundProcessor(unittest.TestCase):

    def test_tg_inbound_processor(self):
        bot = get_tg_bot()

        result = bot.handle(TGTestMessage().get_message())
        self.assertTrue(result.is_ok, result.report())
        self.assertTrue(result.no_replies, result.report())

        result = bot.handle(TGCallbackTestMessage().get_message())
        self.assertTrue(result.is_ok, msg=result.report())
        self.assertTrue(result.no_replies)

    def test_tg_bad_inbound_processor(self):
        bot = get_tg_bot(is_send_process_error_message=False)
        bot.set_inbound_processor(BadInboundProcessor)
        result = bot.handle(TGTestMessage().get_message())
        self.assertFalse(result.is_ok)
        self.assertEqual(BotProcessStatus.unknown_error, result.process_result.status, result.process_result.exception)
        self.assertTrue(result.no_replies)

    def test_tg_bad_inbound_message(self):
        bot = get_tg_bot(is_send_process_error_message=False)
        # empty dictionary
        # with self.assertRaises(KeyError):
        result = bot.handle({})
        self.assertFalse(result.is_ok)
        self.assertEqual(result.process_result.status, BotProcessStatus.inbound_error)
        self.assertTrue(result.no_replies)

    def test_vk_inbound_processor(self):
        bot = get_vk_bot()
        msg = VKTestMessage().get_message(bot_id=bot.config.bot_id)
        result = bot.handle(msg)
        self.assertTrue(result.is_ok, f'Result: {result.report()} // Inbound: {msg}')
        self.assertTrue(result.no_replies)

    def test_vk_secret_process(self):
        bot = get_vk_bot()
        message = VKTestMessage().get_message(bot_id=bot.config.bot_id)

        # bad secret
        message['secret'] = 'bad_secret'
        result = bot.handle(message)
        self.assertEqual(result.process_result.status, BotProcessStatus.wrong_secret, msg=result.report())
        self.assertFalse(result.is_ok)

        # good secret
        message['secret'] = bot.config.webhook_secret
        result = bot.handle(message)
        self.assertEqual(result.process_result.status, BotProcessStatus.ok)
        self.assertTrue(result.is_ok)

    def test_get_name(self):
        class EchoUserDataInboundProcessor(InboundProcessor):
            @staticmethod
            def process(context: BotContext, inbound: InboundMessage):
                text = f'{inbound.user.visual_name} / {inbound.user.short_name}'
                context.replies.add_text(text)

        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                bot.set_inbound_processor(EchoUserDataInboundProcessor)
                message = bot_test.message_getter.get_message(bot_id=bot.config.bot_id)
                inbound = bot._init_inbound(message)
                result = bot._process(inbound)
                self.assertEqual(1, bot.context.replies.count, result.data)
                self.assertEqual(BotProcessStatus.ok, result.status, msg=result.exception)
                self.assertEqual(bot_test.visual_name_text, bot.context.replies[0].text, msg=bot.info.name)

    def test_basic_processor(self):
        """Checking test bot."""
        message_text = f'{MARKER} echo'

        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                bot.set_inbound_processor(TestingInboundProcessor)
                message = bot_test.message_getter.get_message(text=message_text, bot_id=bot.config.bot_id)
                inbound = bot._init_inbound(message)
                self.assertEqual(inbound.context.bot.channel, bot.channel, msg='No match of inbound and bot channel')
                result = bot._process(inbound)
                self.assertEqual(bot.context.replies.count, 1)
                self.assertEqual(result.status, BotProcessStatus.ok, msg=result.exception)
                self.assertEqual(bot.context.replies[0].text, message_text, msg=bot.info.name)

    def test_filters(self):
        class TextMessageFilter(MessageFilter):
            @staticmethod
            def is_true(context: BotContext, message: InboundMessage) -> bool:
                return message.text != 'filter_out'

        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                # NoBotMessage filter
                message = bot_test.message_getter.get_message(user_id=bot.config.bot_id, bot_id=bot.config.bot_id)
                inbound = bot._init_inbound(message)
                result = bot._process(inbound)
                self.assertEqual(0, bot.context.replies.count, msg=bot.context.replies)
                self.assertEqual(BotProcessStatus.filter_not_passed, result.status, msg=result.exception)

                # MessageFilter add
                bot.message_filters_manager.add_filter(TextMessageFilter)
                message = bot_test.message_getter.get_message(text='filter_out', user_id=bot.config.bot_id,
                                                              bot_id=bot.config.bot_id)
                inbound = bot._init_inbound(message)
                result = bot._process(inbound)
                self.assertEqual(bot.context.replies.count, 0)
                self.assertEqual(BotProcessStatus.filter_not_passed, result.status, msg=result.exception)

    def test_bot_manager(self):
        manager = BotsManager()
        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                manager.add(bot)
                bot_from_manager = manager.channels[bot.channel][bot.config.bot_id_string]
                self.assertEqual(bot.config.bot_id, bot_from_manager.config.bot_id)

        manager.set_inbound_processor(InboundProcessor)

    def test_tg_inbound_reply(self):
        bot = get_tg_bot()
        message = TG_MESSAGE_REPLY
        inbound = bot._init_inbound(message)
        self.assertIsNotNone(inbound)
        self.assertIsNotNone(inbound.text)
        self.assertIsNotNone(inbound.reply_to_msg.message_id)
        self.assertTrue(isinstance(inbound.reply_to_msg.user_visual_name, str))
        self.assertTrue(isinstance(inbound.reply_to_msg.date, datetime.datetime))

    def test_vk_inbound_reply(self):
        bot = get_vk_bot()
        message = VK_MESSAGE_REPLY
        inbound = bot._init_inbound(message)
        self.assertIsNotNone(inbound)
        self.assertIsNotNone(inbound.text)
        self.assertIsNotNone(inbound.reply_to_msg.message_id)
        self.assertTrue(isinstance(inbound.reply_to_msg.user_visual_name, str))
        self.assertTrue(isinstance(inbound.reply_to_msg.date_raw, int))
        self.assertTrue(isinstance(inbound.reply_to_msg.date, datetime.datetime))

    def test_dtf_inbound_reply(self):
        bot = get_dtf_bot()
        message = DTF_MESSAGE_REPLY
        inbound = bot._init_inbound(message)
        self.assertIsNotNone(inbound)
        self.assertIsNotNone(inbound.text)
        self.assertIsNotNone(inbound.reply_to_msg.message_id)
        self.assertTrue(isinstance(inbound.reply_to_msg.user_visual_name, str))
        self.assertTrue(isinstance(inbound.date_raw, str), msg=f'date_raw: {inbound.reply_to_msg.date_raw}')
        self.assertTrue(isinstance(inbound.date, datetime.datetime))


if __name__ == '__main__':
    import logging
    from ubotadapter.logger import logger

    logger.addHandler(logging.StreamHandler())
    logger.setLevel(logging.DEBUG)
    unittest.main()



import unittest

from ubotadapter.botmsg import Keyboard
from ubotadapter.inbound import *
from .test_tools import *


class TestKeyboardInboundProcessor(InboundProcessor):
    @staticmethod
    def process(context: BotContext, inbound: InboundMessage):
        buttons = [['button_1', 'button_2'], ['button_3']]
        context.replies.add(
            BotMessage('Choose:', keyboard=Keyboard(buttons=buttons, one_time=True, in_line=True)))
        context.replies.add(
            BotMessage('Choose!', keyboard=Keyboard(buttons=buttons, one_time=True, in_line=False)))


class TestBotsInboundProcessor(unittest.TestCase):
    def test_echo_inbound_processor(self):
        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                bot.set_inbound_processor(EchoInboundProcessor)
                for message_getter in bot_test.all_message_getters:
                    result = bot.handle(message_getter.get_message())
                    self.assertTrue(result.is_ok, msg=result.report())
                    self.assertFalse(result.no_replies)

    def test_reply_inbound_processor(self):
        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                bot.set_inbound_processor(EchoReplyToInboundProcessor)
                result = bot.handle(bot_test.message_getter.get_message(text='ut_reply_to'))
                self.assertTrue(result.is_ok, msg=result.report())
                self.assertFalse(result.no_replies)

    def test_keyboard_inbound_processor(self):
        for bot_test in BOTS_LIBRARY:
            bot = bot_test.bot
            with self.subTest(bot=bot.channel.name):
                bot.set_inbound_processor(TestKeyboardInboundProcessor)
                result = bot.handle(bot_test.message_getter.get_message())
                self.assertTrue(result.is_ok, msg=result.report())
                self.assertFalse(result.no_replies)

    def test_vk_get_name(self):
        bot = get_vk_bot()
        bot.set_inbound_processor(EchoNameInboundProcessor)
        message = VKTestMessage().get_message(bot.config.bot_id)
        result = bot.handle(message)
        self.assertTrue(bot.context.replies.count == 1)
        self.assertTrue(result.is_ok, msg=result.report())

    def test_vk_reply_to_group_chat(self):
        bot = get_vk_bot()
        bot.set_inbound_processor(EchoReplyToInboundProcessor)
        result = bot.handle(VKTestMessage().get_message(text='ut_reply_to', chat_id=2000000001, message_id=1))
        self.assertTrue(result.is_ok, msg=result.report())
        self.assertFalse(result.no_replies)

        result = bot.handle(VKTestMessage().get_message(text='ut_reply_to', message_id=1))
        self.assertTrue(result.is_ok, msg=result.report())
        self.assertFalse(result.no_replies)


if __name__ == '__main__':
    import logging

    logger = logging.getLogger('ubotadapter')
    logger.addHandler(logging.StreamHandler())
    logger.setLevel(logging.DEBUG)
    unittest.main()



from ubotadapter.user import User


class TelegramUser(User):
    @property
    def data(self) -> dict:
        return super()._data

    def get_visual_name(self) -> str:
        first_name = self.data.get('first_name', '')
        last_name = self.data.get('last_name', None)
        if not first_name and not last_name:
            return self.id_url
        name = f'{first_name} {last_name}' if last_name else first_name
        return name

    def get_short_name(self) -> str:
        return self.data.get('first_name', None) or self.id_url

    @property
    def id_url(self) -> str:
        return f'@id{self.data["id"]}'



from __future__ import annotations

from datetime import datetime
from functools import cached_property
from typing import Union

from deteefapi import WebhookComment, DeteefAPI
from deteefapi.webhook import CommentCreator

from ubotadapter import InboundMessage, ReplyToMessage
from .user import DTFUser


class DTFReplyToMessage(ReplyToMessage):
    @property
    def data(self) -> WebhookComment:
        return self._data

    @property
    def text(self) -> Union[str, None]:
        return self.data.text

    @property
    def message_id(self) -> Union[int, str]:
        return self.data.id

    @property
    def chat_id(self) -> Union[int, str]:
        return self.inbound.chat_id

    @property
    def user_id(self) -> Union[int, str]:
        return self.data.creator.id

    @property
    def user_visual_name(self) -> Union[str, None]:
        return self.data.creator.name

    @property
    def date_raw(self) -> None:
        return None

    @property
    def inbound(self) -> DTFInboundMessage:
        return self._inbound


class DTFInboundMessage(InboundMessage):
    _UserClass = DTFUser
    _ReplyMessageClass = DTFReplyToMessage

    @property
    def user_id(self) -> int:
        return self._comment.creator.id

    @property
    def user_id_url(self) -> str:
        return DeteefAPI.get_user_url(user_id=self.user_id, name=self._comment.creator.name)

    @property
    def chat_id(self) -> int:
        return self._comment.content.id

    @property
    def message_id(self) -> int:
        return self._comment.id

    @property
    def secret_code(self) -> str:
        # TODO: implement
        return ''

    @property
    def text(self) -> str:
        return self._comment.text

    @cached_property
    def _comment(self) -> WebhookComment:
        return WebhookComment(self._data.get('data', self._data))

    @property
    def user_data(self) -> CommentCreator:
        return self._comment.creator

    @property
    def date_raw(self) -> Union[int, None]:
        return self._comment._data.get('date')

    def raw_date_converter(self, timestamp: str) -> datetime:
        return datetime.fromisoformat(timestamp)

    @property
    def reply_to_msg_raw(self) -> WebhookComment:
        return self._comment.reply_to
